/* anneal.h */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#ifndef _anneal_h_
#define _anneal_h_

#include <gsl/gsl_siman.h>
#include "minimize.h"

#define N_TRIES 200 // how many points do we try before stepping?
#define ITERS_FIXED_T 2 // how many iterations for each T?
#define STEP_SIZE 1 // step size in random walk (= number of point mutations put into seq after each move)
#define K 1.0 // Boltzmann constant
#define T_INITIAL 200.0 // initial temperature
#define MU_T 1.01 // damping factor for temperature
//#define MU_T 1.05 // test
#define T_MIN 2.0e-5 // min temperature
//#define T_MIN 1.0 // test

typedef struct SA_params {
  class Sequence *ss;
  class DNAparams *DNA;
  class Geom *ideal;
  class Geom *wt;
  struct Options *op;
  unsigned int start; // [start,stop] are one-based inclusive coords of a subsequence
  unsigned int stop; // over which the simulated annealing search is done
} SA_params;

void Anneal(Sequence::Sequence *seq, DNAparams::DNAparams &prms, Geom::Geom &xyz_ideal,
	    Geom::Geom &xyz_wt, struct Options *opt); // main sim. anneal. function

double E1(void *xp); // return the energy of a configuration
double E2(void *xp); // return MINUS the energy of a configuration
void S1(const gsl_rng *r, void *xp, double step_size); // modify current configuration using a random step taken from the generator r
double M1(void *xp, void *yp); // return the distance between two configurations xp and yp
void P1(void *xp); // print the contents of the configuration xp (after translating into seq)

void copy1 (void *source, void *dest);
void *copy_construct1 (void *xp);
void destroy1 (void *xp);

#endif

