/* bstable.h */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#ifndef _bstable_h_
#define _bstable_h_

#include <gsl/gsl_math.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include "sequence.h"
#include "dna_parameters.h"
#include "geometry.h"
#include "vector.h"
#include "matrix.h"
#include <math.h>

int get_basestep_index(char l1, char l2);

// This class reads in an actual table of WO parameters (ie computed by Rosetta).
class BStable {
 private:
  void ReadBStable(std::string const & filename);
  double **params; // Lseq * Nprm table of dinuc prms: [ twist   roll   tilt   shift   slide   rise ]
  char **dinuc_labels; // 2-letter basestep labels
  int *dinuc_nums; // typically one-based basestep number (1st column in file)
  unsigned int Lseq; // number of dinucs in file
  static const unsigned int Nprm = 6; // number of WO prms: [ Twist    Roll    Tilt    Shift    Slide    Rise ]
 public:
  BStable();
  BStable(std::string const & filename);
  BStable(gsl_vector *eq, Sequence::Sequence *seq);
  BStable(DNAparams::DNAparams &prms, Sequence::Sequence *seq);
  ~BStable();
  void WriteBStable(std::string const & filename);
  const unsigned int length(void);
  const double get(int i, int j);
  gsl_matrix* Export(void); // export data into a gsl_matrix (allocated inside!)
};

#endif

