/* matrix.h */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#include <stdlib.h>
#include <math.h>
#include <iostream>
#include "geometry.h"
#include "vector.h"

#ifndef matrix_h_
#define matrix_h_


typedef struct mx_type{
  double data[3][3];
}matrix_type;

class Matrix3{
 public:
  Matrix3();
  Matrix3(const Matrix3&);
  Matrix3(const matrix_type&);
  Matrix3(const Vector &v1, const Vector &v2, const Vector &v3); // 3 vectors make 3 **columns** in the matrix
  ~Matrix3(){};
  void loadvecs(Vector &v1, Vector &v2, Vector &v3); // 3 **columns** in the matrix make 3 vectors
  double value(int,int);
  void set(int,int,double);
  void transpose();
  double Evalue(int);
  Vector Evector(int);
  void RegMultiply(Matrix3 &);
  void TransMultiply(Matrix3 &);
  Vector VecMultiply(Vector &);
  void multiply(double num);
  void subtract(Matrix3 &m);
  int symmetric();
  void Print();
 private:
  void Print_data(matrix_type);
  void Jacobi();
  void Schur(int,int,matrix_type &, matrix_type &);
  double Off(matrix_type);
  matrix_type Matrix_Multiply(matrix_type,matrix_type);
  matrix_type Transpose_Multiply(matrix_type,matrix_type);
  matrix_type data;
  int diagonalized;
  double eigenvalues[3];
  Vector eigenvectors[3];
  static matrix_type I;
};


#endif

