/* options.h */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#ifndef _UTIL_H_
#define _UTIL_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>

// WARNING: option name should NOT be a substring of any other option name (eg -PDB is a substring of -PDB_id so -PDB is not allowed).
// Otherwise the ReadArguments function will mix them up!!

struct Options {
  char *fasta; // use char* for back-compatibility
  std::string geom_wt;
  std::string geom_id;
  std::string PDBid_bp;
  std::string PDBid_mst;
  std::string curv;
  std::string fpdb;
  std::string fgeom;
  std::string minmax;
  bool compute_curv;
  bool mf; // mutate_flanks; used only in the context of simulated annealing
  int start; // [start,stop] are one-based inclusive coords of a subsequence
  int stop;  // over which the simulated annealing search is done
  double w_el,w_sh,w_sum,w_dev; // weights for energy terms
  Options() { fasta = NULL; geom_wt = "WOprms_wt.dat"; geom_id = "none"; PDBid_bp = "ideal_helix_fitted.long.pdb";
              PDBid_mst = "ideal_helix_mst_fitted.long.pdb"; curv = "curvature_ideal.dat";
	      fpdb = "none"; fgeom = "none"; minmax = "min";
              w_el = w_sh = w_sum = w_dev = 1.0; compute_curv = false;
	      start = stop = -1; mf = false; }
};

struct Options *ReadArguments(int &argbase, int argc, char **argv);

#endif

