/* vector.h  */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#include <stdlib.h>
#include <math.h>
#include <iostream>

#ifndef vector_h_
#define vector_h_

class Vector{
  friend Vector operator*(double,const Vector &);
 public:
  Vector(double=0,double=0,double=0);
  Vector(const Vector &);
  static Vector Cross(const Vector &, const Vector &);
  Vector &Normalize();
  Vector &Negate();
  int WithinSqrDist(const Vector &,double);
  double Length();
  Vector operator+(const Vector &);
  Vector operator-(const Vector &);
  double operator*(const Vector &);
  Vector operator*(double);
  void operator+=(const Vector &);
  void operator-=(const Vector &);
  void operator*=(double);
  void operator/=(double);
  void Rotate(Vector R, double theta);
  void ProjectNormal(Vector r);
  void set(double=0,double=0,double=0);
  double x,y,z;
};

// Frame struc:
typedef struct coord {
  Vector a,b,c; // x,y,z unit vectors
  Vector orig; // origin of the coord system
} coord;

std::ostream &operator<<(std::ostream &out, Vector v);
std::istream &operator>>(std::istream &in, Vector v);

#endif
