#!/usr/bin/perl -w

# This script will split a file with multiple fasta sequences into several files
# with one fasta seq per file.
# NOTE: it also gets rid of special characters (currently just '|') in FASTA filenames
# (but not in FASTA seqIDs!).

#Command format: ./seq_splitter.pl orig.file

use strict;
use Bio::SeqIO;
use Bio::Perl;

if (@ARGV != 1) {
  die "Please provide a single file with seqs in FASTA format!\n";
}
my $file_in = $ARGV[0];

my $file_in_io = Bio::SeqIO->newFh(-file => $file_in,
				   -format => "fasta");

#Output seqs separately:
my $cnt = 0;
while (my $seq = <$file_in_io>) {
  my $curID = $seq->id;
  #Get rid of possible special characters in the seqID:
  my @bits = split //,$curID;
  my @final_bits = ();
  foreach my $bit (@bits) {
    push @final_bits,$bit unless ($bit =~ /[\|]/);
  }
  my $newfile = join "",@final_bits;
  $newfile .= ".fna";
  my $file_out_io = Bio::SeqIO->newFh(-file => ">$newfile",
				      -format => "fasta");
  print $file_out_io $seq;
  $cnt++;
}

print "Split $file_in into $cnt separate files ...\n";

