/* fastafile.c */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#include "fastafile.h"

FastaFile::FastaFile()
{
  for (int i=0; i<MAX_NUM_SEQUENCES; i++)
    _sequences[i] = NULL;
  _numSequences = 0;
}

void FastaFile::ReadFasta(char *filename)
{
  FILE *fp = fopen(filename,"r");
  if (fp == NULL) {
    printf("Error reading file %s\n",filename);
    exit(1);
  }

  int numSequences = -1; int ptr;
  char *sequence = new char[Sequence::MAX_SEQ_LENGTH+1];
  char name[1024];
  char prevname[1024];

  int ch = 0;
  while ((ch = fgetc(fp)) != EOF){
    if (ch == '>') {	
      // skip to end of line and copy as name
      int nptr = 0;
      while ((ch = fgetc(fp)) != EOF && ch != '\n' && ch != '\r') name[nptr++] = ch;
      if (ch == EOF) break;
      name[nptr] = 0; 
				// get ready for a new region
      if (numSequences > -1) {  // already seen a sequence ...
	sequence[ptr] = 0;
	_sequences[numSequences++] = new Sequence(sequence,ptr,prevname);
	ptr = 0;
      }
      else {                    // this is the first sequence
	numSequences = 0;
	ptr = 0;
      }
      strcpy(prevname,name);
    }
    if (ch != '\n' && ch != '\r') {
      if (ptr==Sequence::MAX_SEQ_LENGTH) continue;
      sequence[ptr++] = ch;
    }
  }

  if (numSequences > -1) {
    sequence[ptr] = 0;
    _sequences[numSequences++] = new Sequence(sequence,ptr,name);
  }

  _numSequences = numSequences;

  fclose(fp);
  delete [] sequence;
}

FastaFile::~FastaFile()
{
  for (int i=0; i<_numSequences; i++) delete _sequences[i];
}

int FastaFile::Size()
{
  return _numSequences;
}

Sequence *FastaFile::operator[] (int index)
{
  return _sequences[index];
}

void FastaFile::Add(Sequence *seq)
{
  _sequences[_numSequences++] = seq;
}

