/* options.c */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#include "options.h"

/////////////////////////////////////////////////////////////////////
///// SYNOPSIS //////////////////////////////////////////////////////
///// Command-line arguments are read here //////////////////////////
/////////////////////////////////////////////////////////////////////

struct Options *ReadArguments(int &argbase, int argc, char **argv)
{
  struct Options *opt = new struct Options;
  while (argbase < argc && argv[argbase][0]=='-') {
    if (strstr(argv[argbase],"-fasta")) {
      argbase++;
      if (argbase >= argc) { // input FASTA file
	printf("Error: -fasta must be followed by a FASTA file!\n");
	exit(1);
      }
      opt->fasta = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-geom_wt")) { // input table of WO wt prms [optional]
      argbase++;
      if (argbase >= argc) {
	printf("Error: -geom_wt must be followed by a table of wildtype WO prms!\n");
	exit(1);
      }
      opt->geom_wt = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-geom_id")) { // input table of WO ideal prms [optional]
      argbase++;
      if (argbase >= argc) {
	printf("Error: -geom_id must be followed by a table of ideal helix WO prms!\n");
	exit(1);
      }
      opt->geom_id = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-PDBid_bp")) { // input PDB with ideal basepair frames (should be in "long" format!) [optional]
      argbase++;
      if (argbase >= argc) {
	printf("Error: -PDBid_bp must be followed by a PDB file (long format)!\n");
	exit(1);
      }
      opt->PDBid_bp = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-PDBid_mst")) { // input PDB with ideal MST frames (should be in "long" format!) [optional]
      argbase++;
      if (argbase >= argc) {
	printf("Error: -PDBid_mst must be followed by a PDB file (long format)!\n");
	exit(1);
      }
      opt->PDBid_mst = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-curv")) { // file with pre-computed curvature coeffs needed by the superhelical constraint [optional]
      argbase++;
      if (argbase >= argc) {
	printf("Error: -curv must be followed by a binary file with curvature coeffs!\n");
	exit(1);
      }
      opt->curv = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-fpdb")) { // file with output PDB filenames (one per each input seq)
      argbase++;
      if (argbase >= argc) {
	printf("Error: -fpdb must be followed by a file with output PDB filenames!\n");
	exit(1);
      }
      opt->fpdb = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-fgeom")) { // file with output WO prm filenames (one per each input seq)
      argbase++;
      if (argbase >= argc) {
	printf("Error: -fgeom must be followed by a file with output WO prm filenames!\n");
	exit(1);
      }
      opt->fgeom = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-minmax")) { // file with output WO prm filenames (one per each input seq)
      argbase++;
      if (argbase >= argc) {
	printf("Error: -minmax must be followed by a min OR max command!\n");
	exit(1);
      }
      opt->minmax = argv[argbase];
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-w_el")) {
      opt->w_el = atof(argv[++argbase]);
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-w_sh")) {
      opt->w_sh = atof(argv[++argbase]);
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-w_sum")) {
      opt->w_sum = atof(argv[++argbase]);
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-w_dev")) {
      opt->w_dev = atof(argv[++argbase]);
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-compute_curv")) {
      opt->compute_curv = true;
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-mf")) {
      opt->mf = true;
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-start")) {
      opt->start = atoi(argv[++argbase]);
      argbase++;
      continue;
    }
    if (strstr(argv[argbase],"-stop")) {
      opt->stop = atoi(argv[++argbase]);
      argbase++;
      continue;
    }
    printf("Error: option %s not recognized\n",argv[argbase]);
    exit(1);
  }
  
  return opt;
}

