/* dna_parameters.h */

/*****************************************************************
        Copyright by Rockefeller University,
can not be reproduced or distributed without written permission of
the copyright holder.  Version 1.1 of April 2006.

Written by Alexandre Morozov (contact person: morozov@edsb.rockefeller.edu),
and Eric Siggia.

The program DNABEND implements an algorithm for computing elastic energies
for DNA molecules of arbitrary length and sequence. DNA can be constrained
to follow an arbitrary 3D curve, such as a left-handed superhelix found
in nucleosomal core particles.

The DNABEND method and its applications are described in the following
paper:

******************************************************************/

#ifndef _dna_params_h_
#define _dna_params_h_

#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include <string>
#include <iostream>
#include <fstream>

// This class reads in WO potential parameters.
class DNAparams {
 private:
  void ReadMomentsTable (std::string const & filename, double **table, int const dim1, int const dim2);
  void ReadFijTable (std::string const & filename, double ***table);
  static const int DIM1 = 16; // matrix dims
  static const int DIM2 = 6;
  std::string DBpath; // "/home/morozov/rosetta++";
  std::string means_bs_file; // "Equil_AM.mean.dat";
  std::string stddevs_bs_file; // "Equil_AM.stddev.dat";
  std::string Fij_file; // "Fij_AM.dat";
 public:
  DNAparams();
  ~DNAparams();
  double  **means_bs; // 16x6
  double  **stddevs_bs; // 16x6
  double  ***Fij; // 16x6x6
};

#endif

